@@ -198,7 +198,7 @@ scalefilterRenderFilterText (CompScreen *s)
 				  fs->filterInfo->textWidth,
 				  fs->filterInfo->textHeight, 32))
 	{
-	    compLogMessage (d, "scalefilterinfo", CompLogLevelError,
+	    compLogMessage ("scalefilter", CompLogLevelError,
 			    "Bind Pixmap to Texture failure");
 	    XFreePixmap (d->display, fs->filterInfo->textPixmap);
 	    fs->filterInfo->textPixmap = None;
@@ -538,6 +538,7 @@ scalefilterHandleKeyPress (CompScreen *s,
 	    timeout = scalefilterGetTimeout (s);
 	    if (timeout > 0)
 		info->timeoutHandle = compAddTimeout (timeout,
+						      (float) timeout * 1.2,
 						      scalefilterFilterTimeout,
 						      s);
 
@@ -778,7 +779,7 @@ scalefilterInitDisplay (CompPlugin  *p,
 
     fd->textAvailable = checkPluginABI ("text", TEXT_ABIVERSION);
     if (!fd->textAvailable)
-	compLogMessage (d, "scalefilter", CompLogLevelWarn,
+	compLogMessage ("scalefilter", CompLogLevelWarn,
 			"No compatible text plugin found.");
 
     WRAP (fd, d, handleEvent, scalefilterHandleEvent);
@@ -849,6 +850,8 @@ scalefilterFiniScreen (CompPlugin *p,
     UNWRAP (fs, s, paintOutput);
     UNWRAP (fs, ss, setScaledPaintAttributes);
 
+    matchFini (&fs->scaleMatch);
+
     if (fs->filterInfo)
     {
 	ss->currentMatch = fs->filterInfo->origMatch;
